package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.local.UserState
import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.repository.mapper.fromRawList
import com.biganto.visual.roompark.domain.contract.DealContract
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.domain.model.PlanPresetModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */
//


class EstateRepository @Inject constructor(
    private val local: ILocalStore,
    private val api: IRoomParkApi,
    private val db: IDb,
    private val file: FileModule
): DealContract {


    init {
        Timber.d("Estate Repository Created")
    }

    private val getFavoritesApi: Observable<List<EstateEntity>> =
        local.recentUser()
            .flatMap {
                when (it) {
                    is UserState.Authenticated -> db.fetchUser(it.uuid.toInt())
                    else -> throw CustomApiException.NotAuthorizedException()
                }
            }
            .flatMap { user ->
                api.getFavorites(user.authToken)
                    .doOnError(Timber::e)
                    .map { fromRawList(it, ::fromRaw) }
                    .doOnNext {
                        it.forEach { estate ->
                            estate.setFavorite(true)
                            estate.user = user
                        }
                    }
                    .doOnNext(db::blockingUpsert)
            }

    private val getFavoritesDb: Observable<List<EstateEntity>> =
        local.recentUser()
            .flatMap {
                when (it) {
                    is UserState.Authenticated -> db.fetchUser(it.uuid.toInt()).take(1)
                    else -> throw CustomApiException.NotAuthorizedException()
                }
            }
            .flatMap {
                db.getUserFavorites(it.uuid)
                    .doOnError(Timber::e)
                    .toList().toObservable()
            }


    override fun getFavorites(): Observable<List<EstateModel>> {
        return Observable.mergeDelayError(
            arrayListOf(
                getFavoritesApi, getFavoritesDb
            )
        ).map { fromEntity(it, ::fromEntity) }
            .doOnError(Timber::e)
    }


    private fun fetchEstateDb(id: Int) = db.getEstate(id)

    override fun getEstate(estateId: Int): Observable<EstateModel> {
        return fetchEstateDb(estateId).map(::fromEntity)
    }


    fun getPlanTypesApi(estateId: Int) =
        api.getEstatePlanTypes(estateId)
            .doOnNext { Timber.d("raw0 $it") }
            .map { fromRawList(it, ::fromRaw) }
            .map {
                it.onEach { plan ->
                    val e = EstateEntity()
                    e.setId(estateId)
                    plan.estateId = e
                }.toList()
            }
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())


    override fun getPlanTypes(estateId: Int): Observable<List<PlanPresetModel>> =
        Observable.mergeDelayError(
            arrayListOf(getPlanTypesApi(estateId))
        ).map { l -> List(l.size) { fromEntity(l[it]) } }


    private fun getPlanApi(estateId: Int
                           , planId:Int
                           , furniture:Boolean? = null
                           , sizes:Boolean? = null
                           , walls:Boolean? = null
                           , electric:Boolean? = null) =
        api.getDirectPlan(estateId, planId,
            furniture?:false
            ,sizes?:false
            ,walls?:false
            ,electric?:false)
            .map{
                val sFile = getPlanFile(
                        estateId = estateId,
                        planId = planId,
                        furniture = furniture,
                        walls = walls,
                        sizes = sizes,
                        electric = electric)
                file.saveFileToDisk(sFile,it)
                sFile.path
            }
            .subscribeOn(Schedulers.io())

    private fun getPlanFile(estateId: Int
                            , planId:Int
                            , furniture:Boolean? = null
                            , sizes:Boolean? = null
                            , walls:Boolean? = null
                            , electric:Boolean? = null)
    = FileModule.getDirectory(file.context
        ,FileModule.FileDirectory.PlanTypeDir(
            estateId = estateId,
            planId = planId,
            furniture = furniture,
            walls = walls,
            sizes = sizes,
            electric = electric
        ))


    override fun getEmptyPlan(estateId: Int
                         ,planId:Int): Observable<String> =
        Observable.mergeDelayError(
            arrayListOf(getPlanApi(estateId,planId))
        )

    override fun getPlan(estateId: Int
                              ,planId:Int
                              , furniture:Boolean?
                              , sizes:Boolean?
                              , walls:Boolean?
                              , electric:Boolean?): Observable<String> =
        Observable.fromCallable { getPlanFile(
            estateId = estateId,
            planId = planId,
            furniture = furniture,
            walls = walls,
            sizes = sizes,
            electric = electric) }.switchMap {
            if (it.exists()) Observable.just(it.path)
            else getPlanApi(estateId
                ,planId
                , furniture
                , sizes
                , walls
                , electric
            )
        }


//    fun getPlanRequestString(estateId: Int
//                                      , planId:Int
//                                      , furniture:Boolean
//                                      , sizes:Boolean
//                                      , walls:Boolean
//                                      , electric:Boolean
//    ) = api.getDirectPlan(estateId,planId,furniture,sizes,electric).

}
