package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.ArticleEntity
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.repository.mapper.fromRawList
import com.biganto.visual.roompark.domain.contract.FeedsContract
import com.biganto.visual.roompark.domain.model.ArticleModel
import com.biganto.visual.roompark.domain.model.ArticlesPreviewModel
import com.biganto.visual.roompark.domain.model.FeedModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import timber.log.Timber.e
import javax.inject.Inject
import kotlin.math.floor

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class FeedsContractModule @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb
): FeedsContract {

    override fun fetchFeeds(): Observable<List<FeedModel>> =
        fetchAllFeeds().doOnError (::e)

    override fun fetchFeedObservable(feedAlias: String): Observable<ArticlesPreviewModel> =
        fetchArticles(feedAlias)

    override fun fetchFeedObservable(feedAlias: String,pageSize:Int,  startIndex:Int)
            : Observable<ArticlesPreviewModel> = fetchArticles(feedAlias,pageSize,startIndex)

    override fun getArticle(id: Int): Observable<ArticleModel> =
        fetchArticle(id)

    init {
        Timber.d("Feeds Repository Created")
    }


    private fun fetchArticlessApi(feedAlias:String,pageSize:Int,  startIndex:Int)
            : Observable<List<ArticleEntity>>? =api.getArticlesPage(
        feedAlias
        ,pageSize
        ,floor(startIndex.toDouble()/pageSize.toDouble()).toInt()+1
    )
                .doOnNext { Timber.d("raw0 $it") }
                .map { it.items }
                .map { fromRaw(it, feedAlias) }
                .doOnNext { Timber.d("raw enitites $it") }
                .doOnNext(db::blockingUpsert)
                .subscribeOn(Schedulers.io())



    private fun fetchArticlesDb(feedAlias: String,pageSize:Int,  startIndex:Int)
            : Observable<MutableList<ArticleEntity>>? =
        db.fetchArticles(feedAlias, pageSize, startIndex)
            .toList()
            .toObservable()
            .doOnNext { Timber.d("db items: ${it.size}") }
            .subscribeOn(Schedulers.io())

    private fun fetchArticles(feedAlias: String,pageSize:Int = 10,  startIndex:Int = 0)
            : Observable<ArticlesPreviewModel> =
        Observable.mergeDelayError(
            arrayListOf(fetchArticlessApi(feedAlias,pageSize
                ,startIndex),fetchArticlesDb(feedAlias,pageSize,startIndex)
            )
        ).map { fromEntity(feedAlias,it) }


    private val fetchFeedsApi =
        api.getFeeds()
            .doOnNext { Timber.d("raw0 $it") }
            .map{fromRawList(it,::fromRaw)}
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())

    private val fetchFeedsDb =
        db.fetchFeeds()
            .toList()
            .toObservable()
            .subscribeOn(Schedulers.io())

    private fun fetchAllFeeds(): Observable<List<FeedModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchFeedsApi,fetchFeedsDb)
        ).map { fromEntity(it,::fromEntity) }

    private fun fetchArticleApi(id:Int) =
        api.getArticle(id)
            .doOnNext { Timber.d("raw0 $it") }
            .map ( ::fromRaw )
            .flatMap { article ->
                db.getArticle(article.id).observable()
                    .map { articleEntity ->
                        articleEntity.setBody(article.body)
                        articleEntity.setPublished(article.published)
                        articleEntity.photo = article.photo
                        articleEntity.setTitle(article.title)
                        articleEntity
                    }
            }
            .map { arrayListOf(it)}
            .doOnNext(db::blockingUpsert)
            .map { it.first() }
            .subscribeOn(Schedulers.io())

    private fun fetchArticleDb(id:Int) =
        db.getArticle(id).observable()
            .filter{it.body != null}
            .subscribeOn(Schedulers.io())

    private fun fetchArticle(articleId:Int): Observable<ArticleModel> =
        Observable.mergeDelayError(
            arrayListOf(fetchArticleApi(articleId),fetchArticleDb(articleId))
        ).map { fromEntity(it,true) }

}

