package com.biganto.visual.roompark.presentation.screen.albums

import android.graphics.Bitmap
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import androidx.core.os.bundleOf
import androidx.core.widget.NestedScrollView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.AlbumPreviewModel
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.presentation.screen.albums.util.AlbumListAdapter
import com.biganto.visual.roompark.presentation.screen.albums.util.AlbumsHeaderAdapter
import com.biganto.visual.roompark.presentation.screen.photo.PhotoScreenController
import com.biganto.visual.roompark.util.extensions.scaleCenterCrop
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import jp.wasabeef.picasso.transformations.BlurTransformation
import jp.wasabeef.picasso.transformations.ColorFilterTransformation
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_ALBUM_INDEX_KEY = "SELECTED_ALBUM_INDEX"

class AlbumsScreenController :
    BigantoBaseController<AlbumsScreenViewState
            , AlbumsScreen
            , AlbumsScreenPresenter>
   , AlbumsScreen {

    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_ALBUM_INDEX_KEY to id))


    @BindView(R.id.headers_recycler_view)
    lateinit var headersRecyclerView: RecyclerView

    @BindView(R.id.albums_recycler_view)
    lateinit var albumsRecyclerView: RecyclerView

    @BindView(R.id.photo_albums_container)
    lateinit var nestedScrollView: NestedScrollView


    @BindView(R.id.header_album_title)
    lateinit var currentAlbomTitle: MaterialTextView




    private val photosBackgroundTarget = object : com.squareup.picasso.Target {
        override fun onPrepareLoad(placeHolderDrawable: Drawable?) {
//            nestedScrollView.background = placeHolderDrawable
        }

        override fun onBitmapFailed(e: Exception?, errorDrawable: Drawable?) {
            Timber.e(e)
        }

        override fun onBitmapLoaded(bitmap: Bitmap?, from: Picasso.LoadedFrom?) {
            bitmap?.scaleCenterCrop(nestedScrollView)?.let {
                nestedScrollView.background = BitmapDrawable(activity?.resources, it)
            }
        }
    }


    private fun bindRecycler() {
        headersRecyclerView.isNestedScrollingEnabled = false
        headersRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.HORIZONTAL, false)
        headersRecyclerView.adapter = AlbumsHeaderAdapter()
        headersRecyclerView.itemAnimator = null

        if (headersRecyclerView.itemDecorationCount == 0)
            headersRecyclerView.addItemDecoration(
                CeilsDecoration(1
                    , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )

        albumsRecyclerView.isNestedScrollingEnabled = false
        albumsRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        albumsRecyclerView.adapter = AlbumListAdapter()
        albumsRecyclerView.itemAnimator = null
    }

    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bottomNavigation.hide()
        bindRecycler()
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: AlbumsScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerAlbumsScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_ALBUM_INDEX_KEY))
        .inject(this)

//    @Inject
//    lateinit var ac: RoomParkMainActivity

    @Inject
    lateinit var bottomNavigation: IBottomNavigation

    override fun render(viewState: AlbumsScreenViewState) {
        when(viewState){
            is AlbumsScreenViewState.Idle -> render(viewState)
            is AlbumsScreenViewState.AlbumsSelected -> render(viewState)
            is AlbumsScreenViewState.AlbumsListLoaded-> render(viewState)
            is AlbumsScreenViewState.HeaderAlbumChoosed-> render(viewState)
            is AlbumsScreenViewState.PhotoSelected -> render(viewState)
            is AlbumsScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: AlbumsScreenViewState.Idle){

    }

    private fun render(viewState: AlbumsScreenViewState.PhotoSelected){
        router.pushController(RouterTransaction.with(PhotoScreenController(viewState.photoId)))
    }

    private fun render(viewState: AlbumsScreenViewState.SomeError) =
        showError(viewState.exception)

    @Inject
    lateinit var picassoAsync:Picasso

    private fun render(viewState: AlbumsScreenViewState.AlbumsListLoaded) {
        (headersRecyclerView.adapter as AlbumsHeaderAdapter).setItems(
            viewState.list.asSequence().sortedByDescending { it.published }.toList()
        )

        headersRecyclerView.let {
            it.scrollToPosition(
                (it.adapter as AlbumsHeaderAdapter).getItemPosition(viewState.selectedAlbumId)
            )
        }

        viewState.list.first { it.albumId == viewState.selectedAlbumId }.let {
            currentAlbomTitle.text = it.title
            picassoAsync
                .load(it.previewUrl)
                .transform(BlurTransformation(activity, 13, 2))
                .transform(ColorFilterTransformation(0xCC000000.toInt()))
                .into(photosBackgroundTarget)
        }

    }

    private fun render(viewState: AlbumsScreenViewState.HeaderAlbumChoosed) {
        (headersRecyclerView.adapter as AlbumsHeaderAdapter)
            .getItemPosition(viewState.item.albumId).let {
            headersRecyclerView.scrollToPosition(it)
        }

        currentAlbomTitle.text = viewState.item.title
        picassoAsync
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(activity, 13, 2))
            .transform(ColorFilterTransformation(0xCC000000.toInt()))
            .into(photosBackgroundTarget)
    }

    private fun render(viewState: AlbumsScreenViewState.AlbumsSelected){
        (albumsRecyclerView.adapter as AlbumListAdapter).setItems(
            viewState.list.asSequence().sortedByDescending { it.published }.toList()
        )
    }

    override fun onAlbumSelected(): Observable<AlbumPreviewModel>
            = (headersRecyclerView.adapter as AlbumsHeaderAdapter)
        .onItemClicked
        .map { it }
        .debounce (300L,TimeUnit.MILLISECONDS)
        .observeOn(AndroidSchedulers.mainThread())
        .doOnNext { (albumsRecyclerView.adapter as AlbumListAdapter).setItems(arrayListOf())}


    override fun onPhotoSelected(): Observable<PhotoModel>
            = (albumsRecyclerView.adapter as AlbumListAdapter)
        .photoObs
        .map { it }
        .debounce (300L,TimeUnit.MILLISECONDS)
        .observeOn(AndroidSchedulers.mainThread())




    override fun getLayoutId(): Int = R.layout.albums_screen

}