package com.biganto.visual.roompark.presentation.screen.settings.util

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.LayoutRes
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.CachedDataModel
import com.biganto.visual.roompark.domain.model.PushSwitchModel
import com.biganto.visual.roompark.util.extensions.bytesToSize
import com.google.android.material.switchmaterial.SwitchMaterial
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.clicks
import io.reactivex.Observable
import io.reactivex.subjects.PublishSubject
import timber.log.Timber
import kotlin.reflect.KClass
import kotlin.reflect.full.valueParameters
import kotlin.reflect.jvm.javaType

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */

abstract class CommonRecyclerAdapter<VH:CommonViewHolder<M>,M:Any> : RecyclerView.Adapter<VH>() {

    private val onModelClicked = PublishSubject.create<M>()

    val onItemClicked : Observable<M>
        get() = onModelClicked

    protected var list: MutableList<M> = mutableListOf()

    fun setItems(items:List<M>){
        this.list.clear()
        this.list.addAll(items)
        notifyDataSetChanged()
    }

    fun addItems(items:List<M>){
        items.asSequence().filter { !list.contains(it) }.toList().let {uniqueItems->
            this.list.addAll(uniqueItems)
            notifyItemRangeInserted(list.size-uniqueItems.size,uniqueItems.size)
        }
    }


    protected abstract val vhKlazz : KClass<VH>

    @LayoutRes
    protected abstract fun getVhLayout() : Int

    private fun inflateViewHolder(parent:ViewGroup): View =
        LayoutInflater.from(parent.context)
            .inflate(getVhLayout(), parent, false)

    private fun getViewHolderInstance(klazz:KClass<VH>, view:View):VH =
        klazz.constructors.first{ it.valueParameters[0].type.javaType == View::class.java}.call(view)


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): VH =
        getViewHolderInstance(vhKlazz,inflateViewHolder(parent))

    override fun getItemCount(): Int = list.size

    override fun onBindViewHolder(holder: VH, position: Int) {
        holder.bindModel(list[position])
        holder.onClicked()
            .subscribe(onModelClicked)
    }
}

abstract class CommonViewHolder<M:Any>(itemView: View): RecyclerView.ViewHolder(itemView) {

    abstract fun onViewBound(model: M)

    fun bindModel(model: M){
        ButterKnife.bind(this, itemView)
        bindModel = model
        onViewBound(model)
    }

    fun onClicked(): Observable<M> = itemView.clicks().map { bindModel }

    private lateinit var bindModel: M
}

class PushListAdapter : CommonRecyclerAdapter<PushViewHolder,PushSwitchModel>() {
    override val vhKlazz: KClass<PushViewHolder>
        get() = PushViewHolder::class

    override fun getVhLayout(): Int = R.layout.bell_switcher_with_text_viewholder
}


class PushViewHolder(itemView: View) : CommonViewHolder<PushSwitchModel>(itemView) {

    @BindView(R.id.bellSwitcherTitle)
    lateinit var bellTitle:MaterialTextView

    @BindView(R.id.bellSwitch)
    lateinit var switcher:ViewGroup

    override fun onViewBound(model: PushSwitchModel) {
        Timber.d("model is : $model")
        bellTitle.text = model.title
        switcher.findViewById<SwitchMaterial>(R.id.switch1).isChecked = model.switchState
    }
}


class CahcedListAdapter : CommonRecyclerAdapter<CachedViewHolder,CachedDataModel>() {
    override val vhKlazz: KClass<CachedViewHolder>
        get() = CachedViewHolder::class

    override fun getVhLayout(): Int = R.layout.text_description_viewholder
}


class CachedViewHolder(itemView: View) : CommonViewHolder<CachedDataModel>(itemView) {

    @BindView(R.id.descriptionTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.descriptionText)
    lateinit var description:MaterialTextView

    override fun onViewBound(model: CachedDataModel) {
        Timber.d("model is : $model")
        title.text = model.title
        description.text = model.amountBytes.bytesToSize()
    }
}

